/*
 * IpatchContainer.h - Patch item container base class
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_CONTAINER_H__
#define __IPATCH_CONTAINER_H__

#include <stdarg.h>
#include <glib.h>
#include <glib-object.h>

/* forward type declarations */

typedef struct _IpatchContainer IpatchContainer;
typedef struct _IpatchContainerClass IpatchContainerClass;

#include <libinstpatch/IpatchItem.h>
#include <libinstpatch/IpatchIter.h>
#include <libinstpatch/IpatchList.h>

#define IPATCH_TYPE_CONTAINER   (ipatch_container_get_type ())
#define IPATCH_CONTAINER(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_CONTAINER, IpatchContainer))
#define IPATCH_CONTAINER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_CONTAINER, \
  IpatchContainerClass))
#define IPATCH_IS_CONTAINER(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_CONTAINER))
#define IPATCH_IS_CONTAINER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_CONTAINER))
#define IPATCH_CONTAINER_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), IPATCH_TYPE_CONTAINER, \
  IpatchContainerClass))

/**
 * IpatchContainerAddHook:
 * @container: Container item
 * @item: Item that was added to @container
 *
 * A function prototype to hook container item adds. When assigned to
 * #IpatchContainerClass <structfield>add_hook</structfield> method -
 * this function will be called after an item is added to any
 * container.
 *
 * MT-NOTE: This function is called within @container item write lock,
 * so time spent should be minimized, and care must be taken when locking
 * other items (to prevent dead locks).
 */
typedef void (*IpatchContainerAddHook)(IpatchContainer *container,
				       IpatchItem *item);
/**
 * IpatchContainerRemoveHook:
 * @container: Container item
 * @item: Item being removed
 *
 * A function prototype to hook container item removes. When assigned
 * to #IpatchContainerClass <structfield>remove_hook</structfield>
 * method - this function will be called before every container remove.
 */
typedef void (*IpatchContainerRemoveHook)(IpatchContainer *container,
					  IpatchItem *item);

/* Base patch container object */
struct _IpatchContainer
{
  IpatchItem parent_instance;
};

struct _IpatchContainerClass
{
  IpatchItemClass parent_class;

  /*< public >*/

  /* methods */
  const GType * (*child_types)(void);
  const GType * (*virtual_types)(void);
  gboolean (*init_iter)(IpatchContainer *container, IpatchIter *iter,
			GType type);
  void (*make_unique)(IpatchContainer *container, IpatchItem *item);
  gboolean (*get_dups)(IpatchContainer *container, IpatchItem *item,
		       IpatchList **list);

  /* virtual function hooks */
  IpatchContainerAddHook add_hook;
  IpatchContainerRemoveHook remove_hook;
};

/* container uses no item flags */
#define IPATCH_CONTAINER_UNUSED_FLAG_SHIFT  IPATCH_ITEM_UNUSED_FLAG_SHIFT

#define ipatch_container_append(container, item) \
    ipatch_container_insert (container, item, -1)
#define ipatch_container_prepend(container, item) \
    ipatch_container_insert (container, item, 0)
#define ipatch_container_add  ipatch_container_append

#define IPATCH_CONTAINER_ERRMSG_INVALID_CHILD_2 \
    "Invalid child type '%s' for parent type '%s'"

GType ipatch_container_get_type (void);

IpatchList *ipatch_container_get_children (IpatchContainer *container,
					   GType type);
const GType *ipatch_container_get_child_types (IpatchContainer *container);
const GType *ipatch_container_get_virtual_types (IpatchContainer *container);
const GType *ipatch_container_type_get_child_types (GType container_type);
void ipatch_container_insert (IpatchContainer *container,
			      IpatchItem *item, int pos);
void ipatch_container_remove (IpatchContainer *container, IpatchItem *item);
void ipatch_container_remove_all (IpatchContainer *container);
guint ipatch_container_count (IpatchContainer *container, GType type);
void ipatch_container_make_unique (IpatchContainer *container,
				   IpatchItem *item);
void ipatch_container_add_unique (IpatchContainer *container,
				  IpatchItem *item);
gboolean ipatch_container_init_iter (IpatchContainer *container,
				     IpatchIter *iter, GType type);
void ipatch_container_insert_iter (IpatchContainer *container,
				   IpatchItem *item, IpatchIter *iter);
void ipatch_container_remove_iter (IpatchContainer *container,
				   IpatchIter *iter);

void ipatch_container_set_add_hook (IpatchContainerAddHook hook_func);
void ipatch_container_set_remove_hook (IpatchContainerRemoveHook hook_func);

#endif
