/*
 * IpatchCram.h - Cram common header
 * Compress hybRid Audio Media (instrument compression, etc)
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */

/*
 * Cram
 *
 * For losslessly compressing files that contain audio and binary data
 * such as instrument patch files like SoundFont, DLS, GigaSampler,
 * etc.
 *
 * Format of a Cram file
 * ------------------------
 * The '+' symbol is used to indicate required fields
 * The "<-->" sequence means additional fields/chunks may be added in future
 * versions, so unknown chunks should be skipped (optionally display warning).
 * Changes to format will be indicated with (vN) such as (v2) for version 2.
 * '...' indicates repeated chunks (any chunks in the repeated section).
 *
 * A CRAM file is based on EBML except CRC32 checksums are NOT present, since
 * MD5 is used instead.
 *
 * Field name            EBML ID      data type   default/notes
 * ----------------------------------------------------------------
 * +EBML signature       [1A][45][DF][A3]
 *   +EBMLVersion        [42][86]     uint      1
 *   +EBMLReadVersion    [42][F7]     uint      1
 *   +DocType            [42][82]     string    "CRAM"
 *   +DocTypeVersion     [42][87]     uint      Format version
 *   +DocTypeReadVersion [42][85]     uint      Required decoder version
 *    Flags              [61][31]     uint      0  (IpatchCramFlags)
 *    Software (v2)      [61][3C]     string    Software: "libInstPatch 1.0.0"
 *    Comment (v2)       [61][45]     string    Multi-line comment, use '\n'
 *    <-->
 *
 * +FileChunk            [41][81]     elements
 *   +FileSize           [61][53]     uint - original file size
 *   +FileName           [61][57]     UTF-8 string (relative / delimited path)
 *    FileDate           [61][5C]     uint unix timestamp (seconds since 1970)
 *    Flags                           uint        0  (IpatchCramFlags)
 *    <-->
 *   +FileData           [41][95]     elements
 *      BinaryChunk      [C2]         binary
 *      FlacChunk        [D3]         binary
 *      FlacSplitChunk   [D7]         binary
 *      <Vorbis Chunks> (see below)
 *      <-->
 *      ...
 *   +RelocTable         [41][A5]     compressed elements - relocation table
 *      BinaryChunk      [C2]         elements
 *       +RawSize        [61][63]     uint - Uncompressed data size in bytes
 *        <-->
 *      FlacChunk        [D3]         elements
 *       +RawSize                     uint - Uncompressed data size in bytes
 *        Flags                       uint        0  (IpatchCramFlags)
 *        <-->
 *      FlacSplitChunk   [D7]         elements
 *       +RawSize                     uint - Size of 1 channel in bytes
 *        Flags                       uint        0  (IpatchCramFlags)
 *        ChanCount      [61][65]     1 byte (only 2/stereo supported and is
 *                                    the default - so not usually present)
 *        Offsets        [61][66]     ChanCount-1 uint (offsets from prev chan,
 *                                    defaults to 0 and won't be present then)
 *        <-->
 *      <Vorbis Chunks> (see below)
 *      <-->
 *      ...
 *    RelocTableCkSum    [41][A9]     4 byte zlib adler32 RelocTable checksum
 *    BinaryMD5          [41][AC]     16 byte MD5 sum of binary data
 *    AudioMD5           [41][AF]     Lossless audio MD5, stereo is interleaved
 *    <-->
 * <-->	  (v2) - Future additions here will not be backwards compatible with v1
 * ...
 * +FileEnd              [41][8C]     End of files chunk (EBML size 0)
 *
 *
 * Future vorbis audio support
 * ---------------------------
 * As of this CRAM format version vorbis is not yet supported.  At some point
 * in the future this format will be evaluated for use as a lossy but very
 * high compression audio option.  Below are some pre-liminary chunk
 * definitions that may change once support is added.  This is meant for
 * informational purposes only and is not part of the current format standard,
 * although the chunk IDs should be considered reserved.
 *
 *   +FileData           [41][95]     elements
 *      ...
 *      VorbisChunk      [D9]         binary
 *      VorbisSplitChunk [DD]         binary
 *
 *   +RelocTable         [41][A5]     compressed elements - relocation table
 *      ...
 *      VorbisChunk      [D9]         elements
 *       +RawSize                     uint - Uncompressed data size in bytes
 *        SmoothLoop     [61][73]     2 uints LoopStart, LoopSize
 *        <-->
 *      VorbisSplitChunk [DD]         elements
 *       +RawSize                     uint - Size of 1 channel in bytes
 *        ChanCount                   1 byte (only 2:stereo supported, default)
 *        Offsets                     ChanCount-1 uint (offsets from prev chan)
 *        SmoothLoop                  2 uints LoopStart, LoopSize
 *        <-->
 *
 * MD5 sums
 * --------
 * Two MD5 sums are generated, one for all uncompressed binary data and another
 * for audio. The binary MD5 sum simply uses all concatenated binary data in
 * the order in which it is encoded. The audio MD5 sum is generated from
 * each uncompressed audio sample in the order in which it is encoded, with
 * one caveat being that all stereo data is interleaved before being summed,
 * regardless of whether it is interleaved or not in the original file.
 *
 * Unsigned integer encoding
 * -------------------------
 * All unsigned numbers are encoded EBML style (UTF-8 like encoding). These
 * sizes can be 1 to 8 octets in length to conserve space. They are also
 * stored in big endian byte order. The RelocTableCkSum field is one exception,
 * it is not variable length encoded, but stored as a big endian 4 byte
 * unsigned integer.
 *
 * 0x80               - 0xFE               : 0 to 2^7 - 2
 * 0x4000             - 0x7FFE             : 0 to 2^14 - 2
 * 0x200000           - 0x3FFFFE           : 0 to 2^21 - 2
 * 0x10000000         - 0x1FFFFFFE         : 0 to 2^28 - 2
 * 0x0800000000       - 0x0FFFFFFFFE       : 0 to 2^35 - 2
 * 0x040000000000     - 0x07FFFFFFFFFE     : 0 to 2^42 - 2
 * 0x02000000000000   - 0x03FFFFFFFFFFFE   : 0 to 2^49 - 2
 * 0x0100000000000000 - 0x01FFFFFFFFFFFFFE : 0 to 2^56 - 2
 *
 * Flags
 * -----
 * bit 0-1: Binary compression: 0 = gzip, 1 = reserved for bzip, 2-3 reserved
 * bit 2: Big endian audio flag: 0=little endian, 1=big endian
 * bit 3: Unsigned audio flag: 0=signed, 1=unsigned
 * bit 4-55: Reserved (stored as a variable length integer)
 *
 * Notes
 * -----
 * As of this CRAM format version, there are chunks defined for future lossy
 * Vorbis audio compression support.  They are not part of the current
 * standard and may change when support is officially added.
 *
 * There is a value reserved in the Flags field for bzip compression, but it
 * is not yet a part of the official standard (and may never be depending on
 * how useful it is deemed - binary data is usually only a small portion of
 * the overall size of instrument files).
 *
 * CRAM format version history
 * ---------------------------
 * The CRAM format version is updated whenever any changes occur to the
 * defined format.  If any new features are used in a CRAM file that
 * would cause the parser to break or make it impossible for previous
 * decoder versions to reconstruct the compressed files the appropriate
 * required format read version should be written.  Note that the lowest
 * compatible read version should be used in all cases, so only the use
 * of new features should warrant an increment in the stored read version.
 *
 * v2 - 2005-03-20 - completely backwards compatible with v1
 *    - Added Software and Comment fields to CRAM header
 *    - Future defined chunks should also be expected at the FileChunk
 *        level (backwards compatibility with format v1 will be broken if
 *        a new chunk is added at this level)
 * v1 - First version
 */
#ifndef __IPATCH_CRAM_H__
#define __IPATCH_CRAM_H__

#define IPATCH_CRAM_DOCTYPE_STR  "CRAM"

/* incremented for changes to format (fields added, etc) */
#define IPATCH_CRAM_FORMAT_VERSION	2

/* only change if basic compatibility broken (every variation of CRAM file) */
#define IPATCH_CRAM_FORMAT_READ_VERSION_BASE	1

/* version required if any new chunks are added at the file level */
#define IPATCH_CRAM_FORMAT_READ_VERSION_FILE_LEVEL_CHUNK   2

/* additional format "read" versions for specific features will follow.. */


/* max header size to read to identify a Cram file (up to CRAM signature) */
#define IPATCH_CRAM_MAX_IDENTIFY_SIZE 27

/* minimum EBML header size (all required fields) */
#define IPATCH_CRAM_MIN_HEADER_SIZE 23

#endif
