/*
 * IpatchCramDecoder.h - Cram decoder object
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */
#ifndef __IPATCH_CRAM_DECODER_H__
#define __IPATCH_CRAM_DECODER_H__

#include <glib.h>
#include <libinstpatch/IpatchCram.h>
#include <libinstpatch/IpatchFile.h>
#include <glib-object.h>

typedef struct _IpatchCramDecoder IpatchCramDecoder; /* private structure */
typedef struct _IpatchCramDecoderClass IpatchCramDecoderClass;
typedef struct _IpatchCramDecoderFileInfo IpatchCramDecoderFileInfo;

/* defined in IpatchCramDecoder.c */
typedef struct _DecoderRelocEntry DecoderRelocEntry;

#define IPATCH_TYPE_CRAM_DECODER   (ipatch_cram_decoder_get_type ())
#define IPATCH_CRAM_DECODER(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_CRAM_DECODER, \
   IpatchCramDecoder))
#define IPATCH_CRAM_DECODER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_CRAM_DECODER, \
   IpatchCramDecoderClass))
#define IPATCH_CRAM_IS_DECODER(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_CRAM_DECODER))
#define IPATCH_CRAM_IS_DECODER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_CRAM_DECODER))

/* Cram decoder file info structure */
struct _IpatchCramDecoderFileInfo
{
  char *filename;     /* file name (can be contain a relative path) */
  guint64 size;			/* size of original file */
  guint64 compress_size;    /* total compressed size (incl. CRAM overhead) */
  guint32 date;		     /* unix file timestamp or 0 if not set */
  guint flags;			/* per file flags */
};

/* Cram decoder object */
struct _IpatchCramDecoder
{
  GObject parent_instance;
  int stage;			/* current stage */
  GError *error;		/* instance error pointer */
  IpatchFile *cramfile;		/* cram file being decoded */

  GArray *ebml_chunks;	   /* open EBML chunk array (IpatchCramEbmlChunk) */

  /* variables for current file being decoded */

  guint file_flags;		/* IpatchCramFlags for the current file */

  IpatchCramDecoderFileInfo file_info;
  IpatchFile *destfile;		/* current destination file object */

  gpointer bin_md5ctx;	      /* binary MD5 context of current file */
  gpointer aud_md5ctx;	       /* audio MD5 context of current file */

  GArray *reloc_table; /* relocation table (DecoderRelocEntry) */

  DecoderRelocEntry *bin_entry;	/* current binary relocation entry */
  guint64 bin_entry_pos; /* pos in output file of start of binary entry */
  guint64 bin_entry_ofs;    /* offset in current binary reloc entry */

  DecoderRelocEntry *aud_entry; /* current audio reloc entry (FLAC/Vorbis) */
  guint64 aud_entry_pos; /* pos in output file of start of audio entry */
  guint aud_left; /* remaining uncompressed audio for current entry */

  /* end per file variables */

  guint64 data_remaining; /* remaining compressed data for current segment */

  gpointer zstream;		/* zlib inflate instance */
  gboolean zstream_initialized;	/* TRUE if zstream is initialized */
  gpointer bin_inbuf;	   /* binary input buffer (compressed data) */
  gpointer bin_outbuf;	/* binary output buffer (uncompressed data) */
  gpointer bin_curpos;		/* current position in bin_outbuf */
  guint bin_left;		/* remaining data in bin_outbuf */
  guint64 bin_offset;		/* total uncompressed binary offset */

  gpointer flac;		/* flac decoder instance */
  gboolean forged_streaminfo; /* set after forging FLAC STREAMINFO header */
  gpointer audio_buf;		/* audio conversion buffer */
  guint audio_buf_size;		/* audio buffer size in bytes */

  /* CRAM header fields */
  guint cram_version;		/* CRAM format version used to write file */
  guint cram_read_version;	/* minimum CRAM format version required */
  guint flags;		   /* IpatchCramFlags field of loaded CRAM header */
  char *software;		/* software used to write the file */
  char *comment;		/* comment field of header */

  /* the following variables are set (over written) for each loaded file */
  guint8 binary_md5[16];	/* binary MD5 sum of current file */
  gboolean binary_md5_set;	/* TRUE when binary MD5 is set */
  guint8 audio_md5[16];		/* audio MD5 sum of current file */
  gboolean audio_md5_set;	/* TRUE when audio MD5 is set */
};

/* Cram decoder class */
struct _IpatchCramDecoderClass
{
  GObjectClass parent_class;
};

GType ipatch_cram_decoder_get_type (void);
IpatchCramDecoder *ipatch_cram_decoder_new (IpatchFile *cramfile);
void ipatch_cram_decoder_set_cram_file (IpatchCramDecoder *decoder,
					IpatchFile *cramfile);
gboolean ipatch_cram_decoder_start (IpatchCramDecoder *decoder, GError **err);
IpatchCramDecoderFileInfo *
ipatch_cram_decoder_next_file (IpatchCramDecoder *decoder, GError **err);
void ipatch_cram_decoder_set_dest_file (IpatchCramDecoder *decoder,
					IpatchFile *dest);
gboolean ipatch_cram_decoder_decode_file (IpatchCramDecoder *decoder,
					  GError **err);
gboolean ipatch_cram_decoder_decode_all (IpatchCramDecoder *decoder,
					 const char *path, GError **err);
#endif
