/*
 * IpatchCramEncoder.h - Cram encoder object
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */
#ifndef __IPATCH_CRAM_ENCODER_H__
#define __IPATCH_CRAM_ENCODER_H__

/*
 * Order of operations for encoding Cram files:
 * ipatch_cram_encoder_new (file)
 * ipatch_cram_encoder_set_file (file) - If not done in previous step
 * ipatch_cram_encoder_start ()
 * <FileLoop> - For each encoded file (typically only 1)
 *   ipatch_cram_encoder_declare_file () - Declare the file being encoded
 *   <SegLoop> - Encode binary and audio segments in order of position
 *      ipatch_cram_encoder_declare_binary ()
 *        || ipatch_cram_encoder_declare_audio ()
 *      <DataLoop> - Encode the declared data for this segment
 *        ipatch_cram_encoder_store_binary ()
 *          || ipatch_cram_encoder_store_audio
 *          || ipatch_cram_encoder_store_audio_interleaved ()
 *      </DataLoop>
 *   </SegLoop>
 * </FileLoop>
 * ipatch_cram_encoder_end ()
 */

#include <glib.h>
#include <libinstpatch/IpatchCram.h>
#include <libinstpatch/IpatchFile.h>
#include <glib-object.h>

typedef struct _IpatchCramEncoder IpatchCramEncoder; /* private structure */
typedef struct _IpatchCramEncoderClass IpatchCramEncoderClass;

#define IPATCH_TYPE_CRAM_ENCODER   (ipatch_cram_encoder_get_type ())
#define IPATCH_CRAM_ENCODER(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_CRAM_ENCODER, \
   IpatchCramEncoder))
#define IPATCH_CRAM_ENCODER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_CRAM_ENCODER, \
   IpatchCramEncoderClass))
#define IPATCH_CRAM_IS_ENCODER(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_CRAM_ENCODER))
#define IPATCH_CRAM_IS_ENCODER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_CRAM_ENCODER))

/* Cram encoder object */
struct _IpatchCramEncoder
{
  GObject parent_instance;
  IpatchFile *file;
  gpointer compress_instance;	/* compressor instance */
  gpointer flac_instance;	/* flac instance */
  int stage;			/* current stage */
  GError *error;		/* instance error pointer */
  guint flags;			/* flags (defined in IpatchCramEncoder.c) */
  char *comment;		/* CRAM comment field */

  guint64 file_chunk_ofs; /* ofs to cur EBML file chunk to fixup or 0 */
  guint64 data_chunk_ofs; /* ofs to cur EBML file data chunk to fixup or 0 */
  guint64 binary_chunk_ofs;	/* binary chunk size to fixup or 0 */
  guint64 cur_chunk_ofs; /* current chunk size to fixup (not binary) or 0 */

  gpointer bin_md5ctx;		/* MD5 context for binary */
  gpointer aud_md5ctx;		/* MD5 context for audio */
  gboolean any_binary; /* set to TRUE if any binary data has been saved */
  gboolean any_audio; /* set to TRUE if any lossless audio has been saved */

  gpointer binbuf;		/* binary compression buffer */
  gpointer convbuf;		/* audio conversion buffer */

  GArray *reloc_table;		/* relocation table */
  GList *audio_splits; /* stereo audio splits to resolve (SplitBag) */
  int chunktype; /* current chunk type (0=none, 1=binary, 2=FLAC, etc) */
  guint64 data_remaining; /* remaining data for current seg (bytes/frames) */
  guint64 curpos; /* current position in uncompressed data being encoded */


  /*< public >*/
  guint8 binary_md5[16];    /* final MD5 sum of uncompressed binary */
  guint8 audio_md5[16];	     /* final MD5 sum of uncompressed audio */
  guint64 compress_binary_size;	/* total compressed binary data size */
  guint64 original_size;	/* original file size */
  guint64 total_size;	      /* total size of Cram compressed file */

  /* binary compression parameters */
  int compress_level; /* binary compression level (0-9, -1 = default) */

  /* FLAC audio compression parameters */
  gboolean adaptive_mid_side_coding; /* T:adaptive, F:exhaustive, def:F */
  int block_size;		/* FLAC block size, def:4608 */
  int max_lpc_order;	  /* max LPC order 0-12, 0:only fixed, def:8 */
  int min_rice_partition_order;
  int max_rice_partition_order;
  int qlp_coeff_precision;	/* 0=pick based on blocksize */
  gboolean qlp_coeff_search; /* T:search neighboring coeff, F:fixed */
  gboolean exhaustive_model_search;
};

/* Cram encoder class */
struct _IpatchCramEncoderClass
{
  GObjectClass parent_class;
};

typedef enum  /*< flags >*/
{
  /* audio is split into separate channels (non-interleaved) */
  IPATCH_CRAM_ENCODER_FLAG_SPLIT_AUDIO = 1 << 0,
  IPATCH_CRAM_ENCODER_FLAG_UNSIGNED_AUDIO = 1 << 1 /* audio is unsigned */
} IpatchCramEncoderFlags;

/* structure used for statistics (one entry in relocation table) */
typedef struct
{
  guint64 length;     /* length in bytes (binary) or frames (audio) */
  guint64 offset; /* offset (in bytes) to 2nd channel for split stereo audio */
  guint32 compress_size;  /* resulting compressed size (audio only) */
  guint32 sample_rate;		/* sample rate of audio */
  guint8 channels;		/* 0=binary, 1=mono, 2=stereo */
  guint8 width;			/* 8/16/24 bit audio */
} IpatchCramEncoderStats;

GType ipatch_cram_encoder_get_type (void);
IpatchCramEncoder *ipatch_cram_encoder_new (IpatchFile *file);
void ipatch_cram_encoder_set_file (IpatchCramEncoder *encoder,
				   IpatchFile *file);
void ipatch_cram_encoder_set_audio_endian (IpatchCramEncoder *encoder,
					   gboolean big_endian);
gboolean ipatch_cram_encoder_start (IpatchCramEncoder *encoder, GError **err);
gboolean ipatch_cram_encoder_end (IpatchCramEncoder *encoder, GError **err);
IpatchCramEncoderStats *
ipatch_cram_encoder_get_stats (IpatchCramEncoder *encoder, int *count);

gboolean ipatch_cram_encoder_declare_file (IpatchCramEncoder *encoder,
					   const char *filepath,
					   guint64 size, guint32 datetime,
					   GError **err);
void ipatch_cram_encoder_declare_binary (IpatchCramEncoder *encoder,
					 guint64 length);
void ipatch_cram_encoder_declare_flac_audio (IpatchCramEncoder *encoder,
					     guint64 frames,
					     guint width, guint channels,
					     guint sample_rate, guint flags,
					     guint64 *offsets);
gboolean ipatch_cram_encoder_store_binary (IpatchCramEncoder *encoder,
					   gpointer data,
					   guint length, GError **err);
gboolean ipatch_cram_encoder_store_audio (IpatchCramEncoder *encoder,
					  const gpointer bufs[],
					  guint frames, GError **err);
gboolean
ipatch_cram_encoder_store_audio_interleaved (IpatchCramEncoder *encoder,
					     gconstpointer buf,
					     guint frames, GError **err);
#endif
