/*
 * IpatchFile.h - File type handling object
 * Object that provides file reference counting, MT locking, and
 * a base type for derived file types.
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_FILE_H__
#define __IPATCH_FILE_H__

#include <glib.h>
#include <glib-object.h>

/* forward type declarations */

typedef struct _IpatchFile IpatchFile;
typedef struct _IpatchFileClass IpatchFileClass;
typedef struct _IpatchFileIOFuncs IpatchFileIOFuncs;
typedef struct _IpatchFileBuf IpatchFileBuf;

#include <libinstpatch/IpatchItem.h>

#define IPATCH_TYPE_FILE (ipatch_file_get_type ())
#define IPATCH_FILE(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_FILE, IpatchFile))
#define IPATCH_FILE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_FILE, IpatchFileClass))
#define IPATCH_IS_FILE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_FILE))
#define IPATCH_IS_FILE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_FILE))
#define IPATCH_FILE_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), IPATCH_TYPE_FILE, IpatchFileClass))

/* IO function table for IpatchFile instances */
struct _IpatchFileIOFuncs
{
  gboolean (*open)(IpatchFile *file, const char *mode, GError **err);
  void (*close)(IpatchFile *file);
  GIOStatus (*read)(IpatchFile *file, gpointer buf, guint size,
		    guint *bytes_read, GError **err);
  GIOStatus (*write)(IpatchFile *file, gconstpointer buf, guint size,
		     GError **err);
  GIOStatus (*seek)(IpatchFile *file, int offset, GSeekType type,
		    GError **err);
  int (*getfd)(IpatchFile *file); /* optional get file descriptor method */
  int (*get_size)(IpatchFile *file, GError **err);  /* optional get size method */
};

/* File object */
struct _IpatchFile
{
  IpatchItem parent_instance;

  /*< private >*/

  IpatchFileIOFuncs *iofuncs;	/* per instance I/O methods */
  char *file_name;		/* not always set */
  gint32 pos_counter;		/* a position counter */
  GIOChannel *iochan;		/* glib IO channel (default methods) */
};

/* File object class */
struct _IpatchFileClass
{
  IpatchItemClass parent_class;

  /*< public >*/

  /* signals */
  void (*pos_change)(IpatchFile *file, gint32 offset);

  /* methods */
  gboolean (*identify)(IpatchFile *file);
};

enum
{
  /* is file open? */
  IPATCH_FILE_FLAG_OPEN = 1 << IPATCH_ITEM_UNUSED_FLAG_SHIFT,
  /* swap multi-byte numbers? */
  IPATCH_FILE_FLAG_SWAP = 1 << (IPATCH_ITEM_UNUSED_FLAG_SHIFT + 1),
  /* big endian file? */
  IPATCH_FILE_FLAG_BIG_ENDIAN = 1 << (IPATCH_ITEM_UNUSED_FLAG_SHIFT + 2),
  /* should ->iofuncs be freed? */
  IPATCH_FILE_FLAG_FREE_IOFUNCS = 1 << (IPATCH_ITEM_UNUSED_FLAG_SHIFT + 3),
  /* should file be closed on finalize? */
  IPATCH_FILE_FLAG_CLOSE_ON_FINALIZE = 1 << (IPATCH_ITEM_UNUSED_FLAG_SHIFT + 4)
};

/* reserve 8 flags (3 for expansion) */
#define IPATCH_FILE_UNUSED_FLAG_SHIFT  (IPATCH_ITEM_UNUSED_FLAG_SHIFT + 8)

/* macro to check if file is open */
#define IPATCH_FILE_IS_OPEN(file) \
  (IPATCH_ITEM_FLAGS(file) & IPATCH_FILE_FLAG_OPEN)

/* macro to check if multi-byte numbers in file require swapping */
#define IPATCH_FILE_NEED_SWAP(file) \
  (IPATCH_ITEM_FLAGS(file) & IPATCH_FILE_FLAG_SWAP)
/* macro to check if file is big endian */
#define IPATCH_FILE_BIG_ENDIAN(file) \
  (IPATCH_ITEM_FLAGS(file) & IPATCH_FILE_FLAG_BIG_ENDIAN)

/* run time byte swapping macros for convenient loading of cross endian
   files, takes a pointer to the value to swap (like in a buffer) */
#define IPATCH_FILE_SWAP16(file, p16) \
 (IPATCH_FILE_NEED_SWAP (file) ? \
  GUINT16_SWAP_LE_BE (*(guint16 *)(p16)) : *(guint16 *)(p16))
#define IPATCH_FILE_SWAP32(file, p32) \
 (IPATCH_FILE_NEED_SWAP (file) ? \
  GUINT32_SWAP_LE_BE (*(guint32 *)(p32)) : *(guint32 *)(p32))
#define IPATCH_FILE_SWAP64(file, p64) \
 (IPATCH_FILE_NEED_SWAP (file) ? \
  GUINT64_SWAP_LE_BE (*(guint64 *)(p64)) : *(guint64 *)(p64))

/* structure used for buffered read and writes */
struct _IpatchFileBuf
{
  IpatchFile *file;		/* the file loading/saving from/to */
  gpointer buf;			/* the data buffer */
  int pos;			/* current position in buffer */
  int buf_size;			/* total buffer size */
  int data_size;		/* data size */
  gboolean allocated;		/* set if buffer was allocated */
};

GType ipatch_file_get_type (void);
IpatchFile *ipatch_file_new (void);

void ipatch_file_set_name (IpatchFile *file, const char *file_name);
char *ipatch_file_get_name (IpatchFile *file);
gboolean ipatch_file_open (IpatchFile *file, const char *file_name,
			   const char *mode, GError **err);
gboolean ipatch_file_open_with_fd (IpatchFile *file, int fd, GError **err);
gboolean ipatch_file_open_with_io_channel (IpatchFile *file,
					   GIOChannel *iochan, GError **err);
GIOChannel *ipatch_file_ref_io_channel (IpatchFile *file);
int ipatch_file_get_fd (IpatchFile *file);
gboolean ipatch_file_is_open (IpatchFile *file);
void ipatch_file_close (IpatchFile *file);

int ipatch_file_get_position (IpatchFile *file);
void ipatch_file_update_position (IpatchFile *file, int offset);

gboolean ipatch_file_read (IpatchFile *file, gpointer buf, guint size,
			   GError **err);
GIOStatus ipatch_file_read_eof (IpatchFile *file, gpointer buf, guint size,
				guint *bytes_read, GError **err);
gboolean ipatch_file_write (IpatchFile *file, gconstpointer buf, guint size,
			    GError **err);
#define ipatch_file_skip(file, offset, err) \
  ipatch_file_seek (file, offset, G_SEEK_CUR, err)
gboolean ipatch_file_seek (IpatchFile *file, int offset, GSeekType type,
			   GError **err);
GIOStatus ipatch_file_seek_eof (IpatchFile *file, int offset, GSeekType type,
				GError **err);
int ipatch_file_get_size (IpatchFile *file, GError **err);
GType ipatch_file_identify (IpatchFile *file, GError **err);
IpatchFile *ipatch_file_identify_open (const char *file_name, const char *mode,
				       GError **err);
IpatchFile *ipatch_file_convert_type (IpatchFile *file, GType new_type);

void ipatch_file_set_little_endian (IpatchFile *file);
void ipatch_file_set_big_endian (IpatchFile *file);

gboolean ipatch_file_read_u8 (IpatchFile *file, guint8 *val, GError **err);
gboolean ipatch_file_read_u16 (IpatchFile *file, guint16 *val, GError **err);
gboolean ipatch_file_read_u32 (IpatchFile *file, guint32 *val, GError **err);
gboolean ipatch_file_read_u64 (IpatchFile *file, guint64 *val, GError **err);
gboolean ipatch_file_read_s8 (IpatchFile *file, gint8 *val, GError **err);
gboolean ipatch_file_read_s16 (IpatchFile *file, gint16 *val, GError **err);
gboolean ipatch_file_read_s32 (IpatchFile *file, gint32 *val, GError **err);
gboolean ipatch_file_read_s64 (IpatchFile *file, gint64 *val, GError **err);

gboolean ipatch_file_write_u8 (IpatchFile *file, guint8 val, GError **err);
gboolean ipatch_file_write_u16 (IpatchFile *file, guint16 val, GError **err);
gboolean ipatch_file_write_u32 (IpatchFile *file, guint32 val, GError **err);
gboolean ipatch_file_write_u64 (IpatchFile *file, guint64 val, GError **err);
gboolean ipatch_file_write_s8 (IpatchFile *file, gint8 val, GError **err);
gboolean ipatch_file_write_s16 (IpatchFile *file, gint16 val, GError **err);
gboolean ipatch_file_write_s32 (IpatchFile *file, gint32 val, GError **err);
gboolean ipatch_file_write_s64 (IpatchFile *file, gint64 val, GError **err);

void ipatch_file_buf_init (IpatchFile *file, IpatchFileBuf *filebuf,
			   gpointer buf, guint size);
void ipatch_file_buf_free (IpatchFileBuf *filebuf);
gboolean ipatch_file_buf_load (IpatchFileBuf *filebuf, guint size,
			       GError **err);
void ipatch_file_buf_read (IpatchFileBuf *filebuf, gpointer buf, guint size);
void ipatch_file_buf_write (IpatchFileBuf *filebuf, gconstpointer buf,
			    guint size);
#define ipatch_file_buf_zero(filebuf, size) \
  ipatch_file_buf_memset(filebuf, 0, size)
void ipatch_file_buf_memset (IpatchFileBuf *filebuf, char c, guint size);
gboolean ipatch_file_buf_commit (IpatchFileBuf *filebuf, GError **err);
#define ipatch_file_buf_skip(filebuf, offset) \
  ipatch_file_buf_seek (filebuf, offset, G_SEEK_CUR)
void ipatch_file_buf_seek (IpatchFileBuf *filebuf, int offset, GSeekType type);

guint8 ipatch_file_buf_read_u8 (IpatchFileBuf *filebuf);
guint16 ipatch_file_buf_read_u16 (IpatchFileBuf *filebuf);
guint32 ipatch_file_buf_read_u32 (IpatchFileBuf *filebuf);
guint64 ipatch_file_buf_read_u64 (IpatchFileBuf *filebuf);
gint8 ipatch_file_buf_read_s8 (IpatchFileBuf *filebuf);
gint16 ipatch_file_buf_read_s16 (IpatchFileBuf *filebuf);
gint32 ipatch_file_buf_read_s32 (IpatchFileBuf *filebuf);
gint64 ipatch_file_buf_read_s64 (IpatchFileBuf *filebuf);

void ipatch_file_buf_write_u8 (IpatchFileBuf *filebuf, guint8 val);
void ipatch_file_buf_write_u16 (IpatchFileBuf *filebuf, guint16 val);
void ipatch_file_buf_write_u32 (IpatchFileBuf *filebuf, guint32 val);
void ipatch_file_buf_write_u64 (IpatchFileBuf *filebuf, guint64 val);
void ipatch_file_buf_write_s8 (IpatchFileBuf *filebuf, gint8 val);
void ipatch_file_buf_write_s16 (IpatchFileBuf *filebuf, gint16 val);
void ipatch_file_buf_write_s32 (IpatchFileBuf *filebuf, gint32 val);
void ipatch_file_buf_write_s64 (IpatchFileBuf *filebuf, gint64 val);

void ipatch_file_set_iofuncs_static (IpatchFile *file,
				     IpatchFileIOFuncs *funcs);
void ipatch_file_set_iofuncs (IpatchFile *file,
			      const IpatchFileIOFuncs *funcs);
void ipatch_file_get_iofuncs (IpatchFile *file, IpatchFileIOFuncs *out_funcs);
void ipatch_file_set_iofuncs_null (IpatchFile *file);

gboolean ipatch_file_default_open_method (IpatchFile *file, const char *mode,
					  GError **err);
void ipatch_file_default_close_method (IpatchFile *file);
GIOStatus ipatch_file_default_read_method (IpatchFile *file, gpointer buf,
					   guint size, guint *bytes_read,
					   GError **err);
GIOStatus ipatch_file_default_write_method (IpatchFile *file,
					    gconstpointer buf,
					    guint size, GError **err);
GIOStatus ipatch_file_default_seek_method (IpatchFile *file, int offset,
					   GSeekType type, GError **err);
int ipatch_file_default_getfd_method (IpatchFile *file);
int ipatch_file_default_get_size_method (IpatchFile *file, GError **err);

#endif
