/*
 * IpatchItem.h - Header for IpatchItem base class
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_ITEM_H__
#define __IPATCH_ITEM_H__

#include <stdarg.h>
#include <glib.h>
#include <glib-object.h>

typedef struct _IpatchItem IpatchItem;
typedef struct _IpatchItemClass IpatchItemClass;

#include <libinstpatch/IpatchIter.h>

#define IPATCH_TYPE_ITEM   (ipatch_item_get_type ())
#define IPATCH_ITEM(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_ITEM, IpatchItem))
#define IPATCH_ITEM_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_ITEM, IpatchItemClass))
#define IPATCH_IS_ITEM(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_ITEM))
#define IPATCH_IS_ITEM_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_ITEM))
#define IPATCH_ITEM_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), IPATCH_TYPE_ITEM, IpatchItemClass))

/**
 * IpatchItemPropNotifyHook:
 * @item: Item whose property is being changed
 * @pspec: The parameter spec of the property that is changing
 * @new_value: The new value that will be assigned to the property
 *
 * A function prototype for hooking item property changes. When
 * assigned to the #IpatchItemClass <structfield>prop_change_hook</structfield>
 * method field - this function will be called before every #IpatchItem
 * property change.
 */
typedef void (*IpatchItemPropNotifyHook)(IpatchItem *item,
					 GParamSpec *pspec,
					 const GValue *new_value);

/* Base patch item object */
struct _IpatchItem
{
  GObject object;

  /*< private >*/

  guint32 flags;		/* flag field */
  IpatchItem *parent;	/* parent item or NULL if not parented or root */
  IpatchItem *base; /* base parent object or NULL (for "save" flag updates) */
  GStaticRecMutex *mutex;  /* pointer to mutex (could be a parent's mutex) */
};

/* Base patch item class */
struct _IpatchItemClass
{
  GObjectClass parent_class;

  /*< public >*/

  gboolean mutex_slave;	/* set to TRUE to use parent thread mutex */

  /* methods */
  void (*item_set_property)(GObject *object, guint property_id,
			    const GValue *value, GParamSpec *pspec);
  gboolean (*copy)(IpatchItem *dest, IpatchItem *src, GHashTable *replace,
		   GError **err);
  void (*remove)(IpatchItem *item);

  /* virtual function hook callbacks */

  /* called prior to IpatchItem property change */
  IpatchItemPropNotifyHook pre_prop_notify_hook;

  /* called after IpatchItem property change */
  IpatchItemPropNotifyHook prop_notify_hook;
};

typedef enum /*< flags >*/
{
  IPATCH_ITEM_HOOKS_ACTIVE = 1 << 0,  /* hook callbacks active? */
  IPATCH_ITEM_FREE_MUTEX   = 1 << 1   /* TRUE if item should free its mutex */
} IpatchItemFlags;

/* 2 flags + reserve 2 bits for future expansion */
#define IPATCH_ITEM_UNUSED_FLAG_SHIFT 4

/* non-locking flag macros */
#define IPATCH_ITEM_FLAGS(item)  (((IpatchItem *)(item))->flags)
#define IPATCH_ITEM_SET_FLAGS(item, f) \
  (((IpatchItem *)(item))->flags |= (f))
#define IPATCH_ITEM_CLEAR_FLAGS(item, f) \
  (((IpatchItem *)(item))->flags &= ~(f))

/* Multi-thread locking macros. For now there is no distinction between
   write and read locking since GStaticRWLock is not recursive. */
#define IPATCH_ITEM_WLOCK(item)	\
    g_static_rec_mutex_lock (((IpatchItem *)(item))->mutex)
#define IPATCH_ITEM_WUNLOCK(item) \
    g_static_rec_mutex_unlock (((IpatchItem *)(item))->mutex)
#define IPATCH_ITEM_RLOCK(item)		IPATCH_ITEM_WLOCK(item)
#define IPATCH_ITEM_RUNLOCK(item)	IPATCH_ITEM_WUNLOCK(item)

GType ipatch_item_get_type (void);
guint32 ipatch_item_get_flags (IpatchItem *item);
void ipatch_item_set_flags (IpatchItem *item, int flags);
void ipatch_item_clear_flags (IpatchItem *item, int flags);
void ipatch_item_set_parent (IpatchItem *item, IpatchItem *parent);
void ipatch_item_unparent (IpatchItem *item);
IpatchItem *ipatch_item_ref_parent (IpatchItem *item);
IpatchItem *ipatch_item_peek_parent (IpatchItem *item);
IpatchItem *ipatch_item_ref_base (IpatchItem *item);
IpatchItem *ipatch_item_peek_base (IpatchItem *item);
IpatchItem *ipatch_item_ref_ancestor_type (IpatchItem *item,
					   GType ancestor_type);
IpatchItem *ipatch_item_peek_ancestor_type (IpatchItem *item,
					    GType ancestor_type);
void ipatch_item_remove (IpatchItem *item);
void ipatch_item_changed (IpatchItem *item);
void ipatch_item_prop_notify (IpatchItem *item, GParamSpec *pspec,
			      const GValue *new_value);
void ipatch_item_prop_notify_by_name (IpatchItem *item, const char *prop_name,
				      const GValue *new_value);
gboolean ipatch_item_copy (IpatchItem *dest, IpatchItem *src,
			   GHashTable *replace, GError **err);
IpatchItem *ipatch_item_duplicate (IpatchItem *item, GHashTable *replace,
				   GError **err);

void ipatch_item_set_atomic (gpointer item,
			     const char *first_property_name, ...);
void ipatch_item_get_atomic (gpointer item,
			     const char *first_property_name, ...);
IpatchItem *ipatch_item_first (IpatchIter *iter);
IpatchItem *ipatch_item_next (IpatchIter *iter);

void ipatch_item_set_prop_notify_hook (IpatchItemPropNotifyHook hook_func);

#endif
