/*
 * IpatchPaste.h - Object paste registry and instance
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_PASTE_H__
#define __IPATCH_PASTE_H__

#include <stdarg.h>
#include <glib.h>
#include <glib-object.h>
#include <libinstpatch/IpatchList.h>

/* forward type declarations */

typedef struct _IpatchPaste IpatchPaste;
typedef struct _IpatchPasteClass IpatchPasteClass;

#define IPATCH_TYPE_PASTE   (ipatch_paste_get_type ())
#define IPATCH_PASTE(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_PASTE, IpatchPaste))
#define IPATCH_PASTE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_PASTE, \
   IpatchPasteClass))
#define IPATCH_IS_PASTE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_PASTE))
#define IPATCH_IS_PASTE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_PASTE))
#define IPATCH_PASTE_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), IPATCH_TYPE_PASTE, \
   IpatchPasteClass))

/* paste instance */
struct _IpatchPaste
{
  GObject parent_instance;	/* derived from GObject */

  GObject *src;			/* source object */
  GObject *dest;		/* destination object */
  GType src_type;	/* source type of paste (src type or derived) */
  GType dest_type;	/* destination type of paste (dest type or derived) */

  /* these are set only while a choice is active */
  GObject *choice_src;		/* source object of choice */
  GObject *choice_dest;		/* destination object of choice */
  int choices;			/* available #IpatchPasteChoiceFlags */
};

/* conversion class */
struct _IpatchPasteClass
{
  GObjectClass parent_class;

  /* choice signal, choice parameters are stored in paste instance, signal
     handlers return IpatchPasteChoice value */
  int (*choice)(IpatchPaste *paste);
};

/**
 * IpatchPasteHandler:
 * @paste: Paste object
 * @err: Location to store error information
 *
 * Function prototype for paste handler functions.
 *
 * Returns: %TRUE on success, %FALSE otherwise (in which case an error should
 *   be stored to @err).
 */
typedef gboolean (*IpatchPasteHandler)(IpatchPaste *paste, GError **err);


/* choice values for when a item conflict occurs */
typedef enum
{
  IPATCH_PASTE_CHOICE_NONE,	/* no choice was made */
  IPATCH_PASTE_CHOICE_SKIP,	/* skip item */
  IPATCH_PASTE_CHOICE_REPLACE,	/* replace item */
  IPATCH_PASTE_CHOICE_CHANGED	/* item changed (re-test) */
} IpatchPasteChoice;

/* available choices passed to "choice" signal */
typedef enum
{
  IPATCH_PASTE_CHOICE_FLAG_SKIP    = 1 << 0,
  IPATCH_PASTE_CHOICE_FLAG_REPLACE = 1 << 1,
  IPATCH_PASTE_CHOICE_FLAG_CHANGED = 1 << 2
} IpatchPasteChoiceFlags;

/* flags for ipatch_find_paste_handler() */
typedef enum
{
  IPATCH_PASTE_FIND_DERIVED = 1 << 0   /* find derived types as well */
} IpatchPasteFindFlags;

#define IPATCH_PASTE_SOURCE(paste) ((paste)->src)
#define IPATCH_PASTE_DEST(paste) ((paste)->dest)


void ipatch_register_paste_handler (GType src_type, GType dest_type,
				    IpatchPasteHandler handler);
IpatchPasteHandler ipatch_find_paste_handler (GType src_type, GType dest_type,
					      IpatchPasteFindFlags src_flags,
					      IpatchPasteFindFlags dest_flags);
GType *ipatch_get_paste_dest_types (GType src_type);
GType *ipatch_get_paste_src_types (GType dest_type);

GType ipatch_paste_get_type (void);
IpatchPaste *ipatch_paste_new (void);
void ipatch_paste_set_objects (IpatchPaste *paste, GObject *src,
			       GObject *dest, GType src_type, GType dest_type);
gboolean ipatch_paste_execute (IpatchPaste *paste, GError **err);
gboolean ipatch_paste_default_handler (IpatchPaste *paste, GError **err);

gboolean ipatch_paste_objects (GObject *src, GObject *dest,
			       GType src_type, GType dest_type, GError **err);

gboolean ipatch_paste_is_possible (GType src_type, GType dest_type);
int ipatch_paste_get_choice (IpatchPaste *paste, GObject *src, GObject *dest,
			     int choices);

#endif
