/*
 * IpatchSF2VoiceCache.h - Header for SoundFont voice cache object
 *
 * This is used for pre-processing instruments into arrays of SoundFont
 * compatible voices which can then be accessed very quickly without
 * multi-thread locking or other issues (during synthesis for example).
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_SF2_VOICE_CACHE_H__
#define __IPATCH_SF2_VOICE_CACHE_H__

#include <stdarg.h>
#include <glib.h>
#include <glib-object.h>

#include <libinstpatch/IpatchSampleData.h>
#include <libinstpatch/IpatchSF2Gen.h>
#include <libinstpatch/IpatchSF2Mod.h>

/* forward type declarations */

typedef struct _IpatchSF2VoiceCache IpatchSF2VoiceCache;
typedef struct _IpatchSF2VoiceCacheClass IpatchSF2VoiceCacheClass;
typedef struct _IpatchSF2Voice IpatchSF2Voice;
typedef struct _IpatchSF2VoiceSelInfo IpatchSF2VoiceSelInfo;

#define IPATCH_TYPE_SF2_VOICE_CACHE   (ipatch_sf2_voice_cache_get_type ())
#define IPATCH_SF2_VOICE_CACHE(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_SF2_VOICE_CACHE, \
  IpatchSF2VoiceCache))
#define IPATCH_SF2_VOICE_CACHE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_SF2_VOICE_CACHE, \
  IpatchSF2VoiceCacheClass))
#define IPATCH_IS_SF2_VOICE_CACHE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_SF2_VOICE_CACHE))
#define IPATCH_IS_SF2_VOICE_CACHE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_SF2_VOICE_CACHE))
#define IPATCH_SF2_VOICE_CACHE_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), IPATCH_TYPE_SF2_VOICE_CACHE, \
  IpatchSF2VoiceCacheClass))

/**
 * IpatchSF2VoiceFunc:
 * @cache: SoundFont voice cache
 * @voice: A matching voice
 * @user_data: User defined pointer
 *
 * A function prototype used with ipatch_sf2_voice_cache_foreach(). This
 * function gets called for each matching voice.
 */
typedef void (*IpatchSF2VoiceFunc)(IpatchSF2VoiceCache *cache,
				   IpatchSF2Voice *voice, gpointer user_data);

/* SoundFont voice cache object */
struct _IpatchSF2VoiceCache
{
  GObject parent_instance;

  IpatchSF2VoiceSelInfo *sel_info;	/* array of selection criteria info */
  int sel_count;    /* count of selection ranges per voice (integer pairs) */

  GArray *voices;		/* array of IpatchSF2Voice structures */
  GArray *ranges;	/* array of selection integer pairs for each voice */
};

struct _IpatchSF2VoiceCacheClass
{
  GObjectClass parent_class;
};

/* a SoundFont voice */
struct _IpatchSF2Voice
{
  /* set by SF2VoiceCache converter */
  IpatchSampleData *sample_data; /* sample data for voice */
  guint32 sample_size;		/* size of sample store in frames */
  guint32 loop_start;		/* loop start offset (in samples) */
  guint32 loop_end;   /* loop end offset (in samples, 1st sample after loop) */
  guint32 rate;			/* sample rate */
  guint8 root_note;		/* MIDI root note of sample */
  gint8 fine_tune;		/* fine tune (in cents, -99 - 99) */
  guint16 reserved;		/* reserved (should be 0) */

  IpatchSF2GenArray gen_array;	/* generator effect values */
  GSList *mod_list;	/* modulator list */

  /* set by wavetable implementation */
  IpatchSampleStore *sample_store;  /* optimal sample store to use */

  /* set internally */
  int range_index;  /* index in ranges array (int *) to first selection range */
};

/* voice selection type */
typedef enum
{
  IPATCH_SF2_VOICE_SEL_NOTE,		/* MIDI note range */
  IPATCH_SF2_VOICE_SEL_VELOCITY,	/* MIDI velocity range */
  IPATCH_SF2_VOICE_SEL_AFTER_TOUCH,	/* MIDI aftertouch range */
  IPATCH_SF2_VOICE_SEL_MIDI_CC	/* MIDI custom controller (param1: ctrlnum) */
} IpatchSF2VoiceSelType;

/* selection info structure */
struct _IpatchSF2VoiceSelInfo
{
  IpatchSF2VoiceSelType type;
  int param1;
  int param2;				/* currently not used */
};

/* maximum allowed voice selection criteria (MIDI note, velocity, etc) */
#define IPATCH_SF2_VOICE_CACHE_MAX_SEL_VALUES   64

/* value used for wildcard selection */
#define IPATCH_SF2_VOICE_SEL_WILDCARD (G_MININT)

GType ipatch_sf2_voice_cache_get_type (void);
IpatchSF2VoiceCache *ipatch_sf2_voice_cache_new (IpatchSF2VoiceSelInfo *info,
						 int sel_count);

IpatchSF2Voice *ipatch_sf2_voice_cache_add_voice (IpatchSF2VoiceCache *cache);
void ipatch_sf2_voice_cache_set_voice_range (IpatchSF2VoiceCache *cache,
		IpatchSF2Voice *voice, guint sel_index, int low, int high);
void ipatch_sf2_voice_cache_optimize (IpatchSF2VoiceCache *cache);
void ipatch_sf2_voice_cache_select (IpatchSF2VoiceCache *cache,
				    int *select_values,
				    IpatchSF2Voice **voice_array,
				    guint max_voices);

void ipatch_sf2_voice_convert_sample_data (IpatchSF2Voice *voice,
					   IpatchSampleData *sample_data);
#endif
