/*
 * IpatchSample.h - Header for sample interface.
 * This interface provides a basic API for sample objects.
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */
#ifndef __IPATCH_SAMPLE_H__
#define __IPATCH_SAMPLE_H__

#include <glib.h>
#include <glib-object.h>

#include <libinstpatch/IpatchSampleTransform.h>
#include <libinstpatch/IpatchSampleData.h>

/* forward type declarations */

typedef struct _IpatchSample IpatchSample; /* dummy typedef */
typedef struct _IpatchSampleIface IpatchSampleIface;
typedef struct _IpatchSampleSource IpatchSampleSource;

#define IPATCH_TYPE_SAMPLE   (ipatch_sample_get_type ())
#define IPATCH_SAMPLE(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_SAMPLE, \
   IpatchSample))
#define IPATCH_SAMPLE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_SAMPLE, \
   IpatchSampleIface))
#define IPATCH_IS_SAMPLE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_SAMPLE))
#define IPATCH_SAMPLE_GET_IFACE(obj) \
  (G_TYPE_INSTANCE_GET_INTERFACE ((obj), IPATCH_TYPE_SAMPLE, \
   IpatchSampleIface))

/**
 * IpatchSampleSourceFindFunc:
 * @sample: Sample to find source data in
 * @source: Sample source to initialize
 *
 * #IpatchSample interface method function type to initialize a
 * #IpatchSampleSource structure for a matching sample data
 * source. Some sample objects provide multiple formats and/or storage
 * methods, this method is used to select one source.
 *
 * The @source structure is used for passing options and this method
 * should set any fields it requires in selecting a sample source for later
 * reading.
 *
 * The <structfield>format</structfield> field of the @source is set
 * to the sample format to find (0 as a wildcard), the
 * <structfield>flags</structfield> variable will contain the flag
 * criteria (#IpatchSampleFindFlags).
 *
 * Returns: Should return %TRUE if a source was found, %FALSE otherwise
 */
typedef gboolean (*IpatchSampleSourceFindFunc)(IpatchSample *sample,
					       IpatchSampleSource *source);

/**
 * IpatchSampleSourceReadFunc:
 * @sample: Sample to read data from
 * @source: Sample source
 * @offset: Offset in sample to start read from (in frames)
 * @frames: Size of sample data to read (in frames)
 * @buf: Buffer to store sample data in
 * @err: Location to store error information
 *
 * #IpatchSample interface method function type to read data from a
 * sample.
 *
 * Returns: Should return %TRUE on success, %FALSE otherwise (in which case
 * an error should optionally be stored in @err).
 */
typedef gboolean (*IpatchSampleSourceReadFunc)(IpatchSample *sample,
					       IpatchSampleSource *source,
					       guint offset, guint frames,
					       gpointer buf, GError **err);
/**
 * IpatchSampleSourceCloseFunc:
 * @sample: Object with sample interface
 * @source: Sample source to release resources of
 *
 * Do any cleanup for initialization done in the #IpatchSampleSourceFindFunc
 * method.  This method should free any memory, unref objects, etc that
 * occured in the init phase.
 */
typedef void (*IpatchSampleSourceCloseFunc)(IpatchSample *sample,
					    IpatchSampleSource *source);


/* sample interface */
struct _IpatchSampleIface
{
  GTypeInterface parent_class;

  IpatchSampleSourceFindFunc find;
  IpatchSampleSourceReadFunc read;
  IpatchSampleSourceCloseFunc close;

  guint *loop_types; /* -1 terminated array of supp. loop types (NULL = none) */
};

/* Sample source instance (data1-3 used by IpatchSample interfaces) */
struct _IpatchSampleSource
{
  IpatchSample *sample;		/* IN: original sample interface object */
  IpatchSampleTransform *trans;	/* PRIVATE: if transforming the audio */
  guint16 format;	/* IN: format to search for, OUT: format found */
  guint8 speed;		/* OUT: speed of source (1-10 = slowest-fastest) */
  guint8 flags;		/* IN: find flags (#IpatchSampleFindFlags) */
  gpointer data1;		/* sample interface defined */
  gpointer data2;		/* sample interface defined */
  gpointer data3;		/* sample interface defined */
};

typedef enum
{
  IPATCH_SAMPLE_LOOP_NONE,	/* no loop */
  IPATCH_SAMPLE_LOOP_STANDARD,	/* standard loop */
  IPATCH_SAMPLE_LOOP_RELEASE,	/* loop till release */
  IPATCH_SAMPLE_LOOP_PINGPONG	/* forward then reverse then forward.. */
} IpatchSampleLoopType;

typedef enum
{
  IPATCH_SAMPLE_FIND_FASTEST      = 1 << 0, /* find fastest sample source */
  IPATCH_SAMPLE_FIND_EXACT_FORMAT = 1 << 1 /* only return exact format match */
} IpatchSampleFindFlags;

/* some speed defines as a general guide */
typedef enum
{
  IPATCH_SAMPLE_SPEED_SLOWEST = 1,
  IPATCH_SAMPLE_SPEED_DISK    = 4,
  IPATCH_SAMPLE_SPEED_RAM     = 9,
  IPATCH_SAMPLE_SPEED_FASTEST = 10
} IpatchSampleSpeed;

/* default speed is disk rate */
#define IPATCH_SAMPLE_SPEED_DEFAULT  IPATCH_SAMPLE_SPEED_DISK

/* default root note */
#define IPATCH_SAMPLE_ROOT_NOTE_DEFAULT  60

/* value used for terminating list of supported loop types */
#define IPATCH_SAMPLE_LOOP_TYPE_TERM (-1)

/* some convenient loop_type arrays for use by IpatchSample interfaces */
extern int ipatch_sample_loop_types_standard[];
extern int ipatch_sample_loop_types_standard_release[];

GType ipatch_sample_get_type (void);
int *ipatch_sample_get_loop_types (IpatchSample *sample);
int *ipatch_sample_get_loop_types_by_type (GType type);

gboolean ipatch_sample_find_source (IpatchSample *sample,
				    IpatchSampleSource *source,
				    int format, int flags);
void ipatch_sample_source_set_format (IpatchSampleSource *source, int format);
int ipatch_sample_source_get_format (IpatchSampleSource *source);
int ipatch_sample_source_get_frame_size (IpatchSampleSource *source);
int ipatch_sample_source_get_native_format (IpatchSampleSource *source);
IpatchSampleTransform *
ipatch_sample_source_get_transform (IpatchSampleSource *source);
gpointer ipatch_sample_source_read (IpatchSampleSource *source,
				    guint offset, guint frames,
				    gpointer buf, GError **err);
void ipatch_sample_source_close (IpatchSampleSource *source);

void ipatch_sample_install_property (GObjectClass *oclass, guint property_id,
				     const char *property_name);
void ipatch_sample_install_property_readonly (GObjectClass *oclass,
					      guint property_id,
					      const char *property_name);
GParamSpec *ipatch_sample_new_property_param_spec (const char *property_name,
						   GParamFlags flags);

gboolean
ipatch_sample_find_sample_data_source_method (IpatchSampleData *sampledata,
					      IpatchSampleSource *source);
gboolean
ipatch_sample_read_sample_data_method (IpatchSample *sample,
				       IpatchSampleSource *source,
				       guint offset, guint frames, gpointer buf,
				       GError **err);
void ipatch_sample_close_sample_data_method (IpatchSample *sample,
					     IpatchSampleSource *source);

#endif
