/*
 * IpatchSampleStore.h - Header for sample store base object
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */
#ifndef __IPATCH_SAMPLE_STORE_H__
#define __IPATCH_SAMPLE_STORE_H__

#include <stdarg.h>
#include <glib.h>
#include <glib-object.h>
#include <libinstpatch/IpatchItem.h>
#include <libinstpatch/IpatchSampleTransform.h>
#include <libinstpatch/sample.h>

/* forward type declarations */

typedef struct _IpatchSampleStore IpatchSampleStore;
typedef struct _IpatchSampleStoreClass IpatchSampleStoreClass;

#define IPATCH_TYPE_SAMPLE_STORE (ipatch_sample_store_get_type ())
#define IPATCH_SAMPLE_STORE(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_SAMPLE_STORE, \
  IpatchSampleStore))
#define IPATCH_SAMPLE_STORE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_SAMPLE_STORE, \
  IpatchSampleStoreClass))
#define IPATCH_IS_SAMPLE_STORE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_SAMPLE_STORE))
#define IPATCH_IS_SAMPLE_STORE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_SAMPLE_STORE))
#define IPATCH_SAMPLE_STORE_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), IPATCH_TYPE_SAMPLE_STORE, \
  IpatchSampleStoreClass))

/* sample store base instance */
struct _IpatchSampleStore
{
  IpatchItem parent_instance;
};

/**
 * IpatchSampleStoreVerifyFunc:
 * @store: Sample store to verify
 *
 * #IpatchSampleStore class method function type to verify a sample store's
 * internal state.
 *
 * Returns: Should return %TRUE if sample store's internal state is valid,
 * %FALSE otherwise.
 */
typedef gboolean (*IpatchSampleStoreVerifyFunc)(IpatchSampleStore *store);
/**
 * IpatchSampleStoreAllocFunc:
 * @store: Sample store to allocate for
 * @err: Location to store error information
 *
 * #IpatchSampleStore class method function type to allocate space for a
 * sample store. Function should ensure that data has not already been
 * allocated.
 *
 * Returns: Should return %TRUE on success, %FALSE otherwise (in which case
 * an error should optionally be stored in @err).
 */
typedef gboolean (*IpatchSampleStoreAllocFunc)(IpatchSampleStore *store,
					       GError **err);
/**
 * IpatchSampleStoreFreeFunc:
 * @store: Sample store to free space of
 * @err: Location to store error information
 *
 * #IpatchSampleStore class method function type to free space of a
 * sample store. Function should ensure that there is data to free, etc.
 *
 * Returns: Should return %TRUE on success, %FALSE otherwise (in which case
 * an error should optionally be stored in @err).
 */
typedef gboolean (*IpatchSampleStoreFreeFunc)(IpatchSampleStore *store,
					      GError **err);
/**
 * IpatchSampleStoreReadFunc:
 * @store: Sample store to read data from
 * @offset: Offset in sample to start read from (in frames)
 * @frames: Size of sample data to read (in frames)
 * @buf: Buffer to store sample data in
 * @err: Location to store error information
 *
 * #IpatchSampleStore class method function type to read data from a
 * sample store. The sample store does not need to be verified, this
 * is done by any functions accessing this method. Also the store will
 * already be write locked when control is passed to this method.
 *
 * Returns: Should return %TRUE on success, %FALSE otherwise (in which case
 * an error should optionally be stored in @err).
 */
typedef gboolean (*IpatchSampleStoreReadFunc)(IpatchSampleStore *store,
					      guint offset, guint frames,
					      gpointer buf, GError **err);
/**
 * IpatchSampleStoreWriteFunc:
 * @store: Sample store to write data to
 * @offset: Offset in sample to start write to (in frames)
 * @frames: Size of sample data to write (in frames)
 * @buf: Buffer to read sample data from
 * @err: Location to store error information
 *
 * #IpatchSampleStore class method function type to write data to a
 * sample store. The sample store does not need to be verified, this
 * is done by any functions accessing this method. Also the store will
 * already be write locked when control is passed to this method.
 *
 * Returns: Should return %TRUE on success, %FALSE otherwise (in which case
 * an error should optionally be stored in @err).
 */
typedef gboolean (*IpatchSampleStoreWriteFunc)(IpatchSampleStore *store,
					       guint offset, guint frames,
					       gconstpointer buf,
					       GError **err);
/**
 * IpatchSampleStoreReadableFunc:
 * @store: Sample store to check if readable
 *
 * #IpatchSampleStore class method function type to check if a sample
 * store is readable. This function should only check if a store has reading
 * capability and not verify any other unrelated parameters (i.e., this
 * function is not concerned with whether a read would fail or not, only that
 * this store is read capable).
 *
 * Returns: Should return %TRUE if sample store is readable, %FALSE otherwise.
 */
typedef gboolean (*IpatchSampleStoreReadableFunc)(IpatchSampleStore *store);

/**
 * IpatchSampleStoreWritableFunc:
 * @store: Sample store to check if writable
 *
 * #IpatchSampleStore class method function type to check if a sample
 * store is writable. This function should only check if a store has writing
 * capability and not verify any other unrelated parameters (i.e., this
 * function is not concerned with whether a write would fail or not, only that
 * this store is write capable).
 *
 * Returns: Should return %TRUE if sample store is writable, %FALSE otherwise.
 */
typedef gboolean (*IpatchSampleStoreWritableFunc)(IpatchSampleStore *store);

/**
 * IpatchSampleStoreSpeedFunc:
 * @store: Sample store to get speed rating of
 *
 * #IpatchSampleStore class method function type to get speed rating of a
 * sample store. A value between 1-10 (see #IpatchSampleSpeed enum).
 *
 * Returns: Should return a value between 1 and 10.
 */
typedef guint (*IpatchSampleStoreSpeedFunc)(IpatchSampleStore *store);

/* sample store base class */
struct _IpatchSampleStoreClass
{
  IpatchItemClass parent_class;

  /*< public >*/

  /* methods for derived sample store types */
  IpatchSampleStoreVerifyFunc verify;
  IpatchSampleStoreAllocFunc alloc;
  IpatchSampleStoreFreeFunc free;
  IpatchSampleStoreReadFunc read;
  IpatchSampleStoreWriteFunc write;
  IpatchSampleStoreReadableFunc readable;
  IpatchSampleStoreWritableFunc writable;
  IpatchSampleStoreSpeedFunc speed;
};


/* IpatchSampleWidth | sign | endian | channels stored in flags field
   (7 bits) - see sample.h */


/* flags bit shift value to sample format field */
#define IPATCH_SAMPLE_STORE_FORMAT_SHIFT IPATCH_ITEM_UNUSED_FLAG_SHIFT

/* macro for getting a sample store format enum from flags */
#define IPATCH_SAMPLE_STORE_GET_FORMAT(store) \
  ((IPATCH_ITEM_FLAGS (store) >> IPATCH_SAMPLE_STORE_FORMAT_SHIFT) \
   & IPATCH_SAMPLE_FORMAT_MASK)

/* flags after sample data format field */
typedef enum /*< flags >*/
{
  IPATCH_SAMPLE_STORE_ACTIVE = 1 << (IPATCH_SAMPLE_STORE_FORMAT_SHIFT + IPATCH_SAMPLE_FORMAT_BITCOUNT)
} IpatchSampleStoreFlags;

#define IPATCH_SAMPLE_STORE_IS_ACTIVE(store) \
  (IPATCH_ITEM_FLAGS (store) & IPATCH_SAMPLE_STORE_ACTIVE)

/* we reserve flags for format, active bit and 2 for expansion */
#define IPATCH_SAMPLE_STORE_UNUSED_FLAG_SHIFT \
  (IPATCH_ITEM_UNUSED_FLAG_SHIFT + IPATCH_SAMPLE_FORMAT_BITCOUNT + 1 + 2)

GType ipatch_sample_store_get_type (void);

IpatchSampleStore *ipatch_sample_store_first (IpatchIter *iter);
IpatchSampleStore *ipatch_sample_store_next (IpatchIter *iter);

void ipatch_sample_store_activate (IpatchSampleStore *store);
gboolean ipatch_sample_store_get_active (IpatchSampleStore *store);
void ipatch_sample_store_set_format (IpatchSampleStore *store, int format);
int ipatch_sample_store_get_format (IpatchSampleStore *store);
guint ipatch_sample_store_get_size (IpatchSampleStore *store, guint *bytes);
guint ipatch_sample_store_get_frame_size (IpatchSampleStore *store);

gboolean ipatch_sample_store_is_readable (IpatchSampleStore *store);
gboolean ipatch_sample_store_is_writable (IpatchSampleStore *store);
int ipatch_sample_store_get_speed (IpatchSampleStore *store);

gboolean ipatch_sample_store_alloc (IpatchSampleStore *store, GError **err);
gboolean ipatch_sample_store_read (IpatchSampleStore *store, guint offset,
				   guint frames, void *buf, GError **err);
gboolean ipatch_sample_store_write (IpatchSampleStore *store, guint offset,
				    guint frames, const void *buf,
				    GError **err);

IpatchSampleTransform *
ipatch_sample_store_transform_new (IpatchSampleStore *store,
				   int format, guint frames, gboolean is_read);
IpatchSampleTransform *
ipatch_sample_store_transform_new_size (IpatchSampleStore *store,
					int format, guint *size,
					gboolean is_read);
void ipatch_sample_store_transform_init (IpatchSampleStore *store,
					 IpatchSampleTransform *trans,
					 int format, guint frames,
					 gboolean is_read);
int ipatch_sample_store_transform_init_size (IpatchSampleStore *store,
					     IpatchSampleTransform *trans,
					     int format, guint size,
					     gboolean is_read);
gpointer ipatch_sample_store_transform_read (IpatchSampleStore *store,
					     IpatchSampleTransform *trans,
					     guint offset, guint frames,
					     GError **err);
gboolean ipatch_sample_store_transform_write (IpatchSampleStore *store,
					      IpatchSampleTransform *trans,
					      guint offset, guint frames,
					      GError **err);

gboolean ipatch_sample_store_copy (IpatchSampleStore *dest_store,
				   IpatchSampleStore *src_store,
				   GError **err);
IpatchSampleStore *ipatch_sample_store_duplicate (IpatchSampleStore *store,
						  GType new_type,
						  int new_format,
						  GError **err);

gboolean ipatch_sample_store_TRUE (IpatchSampleStore *store);
gboolean ipatch_sample_store_FALSE (IpatchSampleStore *store);

#endif
