/*
 * IpatchUnit.h - Patch parameter unit conversion registry
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_UNIT_H__
#define __IPATCH_UNIT_H__

#include <glib.h>
#include <glib-object.h>

typedef struct _IpatchUnitInfo IpatchUnitInfo;

/* structure defining a unit type */
struct _IpatchUnitInfo
{
  guint16 id;			/* unit enumeration ID (set internally) */
  guint8 flags;			/* unit flags */
  guint8 digits;	   /* significant digits to display to user */
  GType value_type;		/* unit value type */
  char *name;			/* name identifier */
  char *label;			/* unit label */
  char *descr;			/* unit description */
};

typedef enum /*< flags >*/
{
  IPATCH_UNIT_LOGARITHMIC = 1 << 0, /* unit is logarithmic */
  IPATCH_UNIT_USER = 1 << 1	/* a user friendly unit type */
} IpatchUnitFlags;

/* unit conversion function type */
typedef void (*IpatchUnitConvertFunc)(const GValue *src_val, GValue *dest_val);

/* built-in unit types */
typedef enum
{
  IPATCH_UNIT_TYPE_NONE = 0,	/* a NULL value */
  IPATCH_UNIT_TYPE_INT = 1,
  IPATCH_UNIT_TYPE_UINT = 2,
  IPATCH_UNIT_TYPE_RANGE = 3,
  IPATCH_UNIT_TYPE_DECIBEL = 4,
  IPATCH_UNIT_TYPE_PERCENT = 5,
  IPATCH_UNIT_TYPE_SEMITONE = 6,
  IPATCH_UNIT_TYPE_CENT = 7,
  IPATCH_UNIT_TYPE_SAMPLE_RATE = 8,
  IPATCH_UNIT_TYPE_SAMPLES = 9,
  IPATCH_UNIT_TYPE_HERTZ = 10,
  IPATCH_UNIT_TYPE_SECONDS = 11,

  /* 128 - 159 reserved for IpatchUnit_DLS.h */
  IPATCH_UNIT_TYPE_DLS_GAIN = 128,
  IPATCH_UNIT_TYPE_DLS_ABS_TIME = 129,
  IPATCH_UNIT_TYPE_DLS_ABS_PITCH = 130,
  IPATCH_UNIT_TYPE_TENTH_PERCENT = 131,

  /* 160 - 191 reserved for IpatchUnit_SF2.h */
  IPATCH_UNIT_TYPE_SF2_ABS_PITCH = 160,
  IPATCH_UNIT_TYPE_SF2_ABS_TIME = 161,
  IPATCH_UNIT_TYPE_CENTIBEL = 162,
  IPATCH_UNIT_TYPE_32K_SAMPLES = 163
} IpatchUnitType;


/*
 * Unit class types define domains of conversion, an example is the "user"
 * unit class which is used to convert values to units digestable by a human.
 * A conversion class is essentially a mapping between unit types, which can
 * then be used to lookup conversion functions.
 */
typedef enum
{
  IPATCH_UNIT_CLASS_NONE,	/* a NULL value */
  IPATCH_UNIT_CLASS_USER,   /* "user" conversion class (for humans) */
  IPATCH_UNIT_CLASS_DLS,	/* DLS (native patch type) class */
  IPATCH_UNIT_CLASS_COUNT   /* NOT A VALID CLASS - count of classes */
} IpatchUnitClassType;


IpatchUnitInfo *ipatch_unit_info_alloc (void);
void ipatch_unit_info_free (IpatchUnitInfo *info);
guint16 ipatch_unit_register (const IpatchUnitInfo *info);
IpatchUnitInfo *ipatch_unit_lookup_by_id (guint16 id);
IpatchUnitInfo *ipatch_unit_lookup_by_name (const char *name);
void ipatch_unit_class_register_map (guint16 class_type, guint16 src_units,
				     guint16 dest_units);
guint16 ipatch_unit_class_lookup_map (guint16 class_type, guint16 src_units);
void ipatch_unit_conversion_register (guint16 src_units, guint16 dest_units,
				      IpatchUnitConvertFunc func);
IpatchUnitConvertFunc ipatch_unit_conversion_lookup (guint16 src_units,
						     guint16 dest_units,
						     gboolean *set);
gboolean ipatch_unit_convert (guint16 src_units, guint16 dest_units,
			      const GValue *src_val, GValue *dest_val);
float ipatch_unit_user_class_convert (guint16 src_units,
				      const GValue *src_val);

#endif
