/*
 * sample.h - Header for sample format and transform routines
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */
#ifndef __SAMPLE_H__
#define __SAMPLE_H__

#include <stdio.h>
#include <glib.h>
#include <glib-object.h>
#include <libinstpatch/IpatchSampleTransform.h>

/* sample formats are specified by IpatchSampleWidth | IpatchSampleSign
   | IpatchSampleEndian | IpatchSampleChannel */


#define IPATCH_SAMPLE_WIDTH_MASK    0x07
#define IPATCH_SAMPLE_CHANNEL_MASK  0x18
#define IPATCH_SAMPLE_SIGN_MASK     0x20
#define IPATCH_SAMPLE_ENDIAN_MASK   0x40

#define IPATCH_SAMPLE_ROUTE_MASK    0x80 /* not normally part of format */

#define IPATCH_SAMPLE_WIDTH_SHIFT   0
#define IPATCH_SAMPLE_CHANNEL_SHIFT 3
#define IPATCH_SAMPLE_SIGN_SHIFT    5
#define IPATCH_SAMPLE_ENDIAN_SHIFT  6

#define IPATCH_SAMPLE_ROUTE_SHIFT   7 /* not normally part of format */

/* sample data widths */
typedef enum
{
  IPATCH_SAMPLE_INVALID = 0, /* A NULL value */
  IPATCH_SAMPLE_8BIT  = 1, /* 8 bit integer PCM */
  IPATCH_SAMPLE_16BIT = 2, /* 16 bit integer PCM */
  IPATCH_SAMPLE_24BIT = 3, /* 24 bit integer PCM (32 bit ints) */
  IPATCH_SAMPLE_32BIT = 4, /* 32 bit integer PCM */
  IPATCH_SAMPLE_FLOAT = 5, /* 32 bit IEEE float (0.0 - 1.0) */
  IPATCH_SAMPLE_DOUBLE = 6, /* 64 bit IEEE double (0.0 - 1.0) */
  IPATCH_SAMPLE_REAL24BIT = 7 /* real 3 byte 24 bit data */
} IpatchSampleWidth;

/* sample channels */
typedef enum
{
  IPATCH_SAMPLE_MONO    = 0 << IPATCH_SAMPLE_CHANNEL_SHIFT, /* mono audio */
  IPATCH_SAMPLE_STEREO  = 1 << IPATCH_SAMPLE_CHANNEL_SHIFT /* stereo audio */
} IpatchSampleChannel;

typedef enum
{
  IPATCH_SAMPLE_SIGNED   = 0 << IPATCH_SAMPLE_SIGN_SHIFT, /* data is signed */
  IPATCH_SAMPLE_UNSIGNED = 1 << IPATCH_SAMPLE_SIGN_SHIFT /* data is unsigned */
} IpatchSampleSign;

typedef enum
{
  IPATCH_SAMPLE_LENDIAN = 0 << IPATCH_SAMPLE_ENDIAN_SHIFT, /* little endian */
  IPATCH_SAMPLE_BENDIAN = 1 << IPATCH_SAMPLE_ENDIAN_SHIFT /* big endian flag */
} IpatchSampleEndian;

/* define convenience host byte order value */
#if G_BYTE_ORDER == G_LITTLE_ENDIAN
#define IPATCH_SAMPLE_ENDIAN_HOST IPATCH_SAMPLE_LENDIAN
#else
#define IPATCH_SAMPLE_ENDIAN_HOST IPATCH_SAMPLE_BENDIAN
#endif

/* channel routing for sample conversion from stereo to mono, used by some
   conversion functions */
typedef enum
{
  IPATCH_SAMPLE_ROUTE_LEFT  = 0 << IPATCH_SAMPLE_ROUTE_SHIFT,
  IPATCH_SAMPLE_ROUTE_RIGHT = 1 << IPATCH_SAMPLE_ROUTE_SHIFT
} IpatchSampleChannelRoute;

/* mask for all fields of sample format (width, sign, endian, channel) */
#define IPATCH_SAMPLE_FORMAT_MASK   0x7F

/* number of bits used for sample format (not including channel routing) */
#define IPATCH_SAMPLE_FORMAT_BITCOUNT  7

/* get IpatchSampleWidth enum from a sample format integer */
#define IPATCH_SAMPLE_FORMAT_GET_WIDTH(format) \
  ((format) & IPATCH_SAMPLE_WIDTH_MASK)

#define IPATCH_SAMPLE_FORMAT_IS_FLOATING(format) \
  (((format) & IPATCH_SAMPLE_WIDTH_MASK) == IPATCH_SAMPLE_FLOAT \
   || ((format) & IPATCH_SAMPLE_WIDTH_MASK) == IPATCH_SAMPLE_DOUBLE)

/* macro to get channel count from sample format */
#define IPATCH_SAMPLE_FORMAT_GET_CHANNELS(format) \
  ((((format) & IPATCH_SAMPLE_CHANNEL_MASK) >> IPATCH_SAMPLE_CHANNEL_SHIFT) + 1)

/* macros for checking sign */
#define IPATCH_SAMPLE_FORMAT_IS_SIGNED(format) \
  (((format) & IPATCH_SAMPLE_UNSIGNED) == 0)
#define IPATCH_SAMPLE_FORMAT_IS_UNSIGNED(format) \
  (((format) & IPATCH_SAMPLE_UNSIGNED) != 0)

/* macros for checking endianness */
#define IPATCH_SAMPLE_FORMAT_IS_LENDIAN(format) \
  (((format) & IPATCH_SAMPLE_BENDIAN) == 0)
#define IPATCH_SAMPLE_FORMAT_IS_BENDIAN(format) \
  (((format) & IPATCH_SAMPLE_BENDIAN) != 0)

/* using a macro for extra speed, also cheating with IPATCH_SAMPLE_STEREO
   and using it like a flag */
#define ipatch_sample_format_size(format) \
  (ipatch_sample_width_sizes[(format) & IPATCH_SAMPLE_WIDTH_MASK] << \
   (((format) & IPATCH_SAMPLE_STEREO) ? 1 : 0))

extern guint8 ipatch_sample_width_sizes[8];

gboolean ipatch_sample_format_verify (int format);
IpatchSampleTransformFunc *ipatch_sample_new_transform_func_array
(int src_format, int dest_format, guint *buf1_max_frame,
 guint *buf2_max_frame, guint *func_count);

#endif
